==> 1.6.1&file=out_yasapi-1.6.1-src.tar.gz: out_yasapi-1.6.1/out_yasapi/yasapi_control_list.c <==
/*
 * yasapi_config_list.c
 * Copyright (C) 2015-2016 Peter Belkner <pbelkner@snafu.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <yasapi.h>
#include <resources.h>

///////////////////////////////////////////////////////////////////////////////
#if defined (YASAPI_DEBUG) // {
static const ControlComboBoxList gcaDebugList[]={
  { L"Off",0 },
  { L"Default",1 },
  { L"Verbose 1",2 },
  { L"Verbose 2",3 },
  { L"Verbose 3",4 },
  { NULL,0 }
};

static const ControlComboBoxConfig gcDebugConfig=
  { IDC_COMBOBOX_DEBUG,offsetof(OptionsCommon,nDebug),gcaDebugList,
      L"Whether to switch off the debug console or whether to write"
      L" in default or verbose mode to it." };

////////////////
static const ControlComboBoxList gcaSleepList[]={
  { L"0 sec",0 },
  { L"2 sec",2*1000 },
  { L"5 sec",5*1000 },
  { L"10 sec",10*1000 },
  { L"20 sec",20*1000 },
  { L"1 min",1*60*1000 },
  { L"2 min",2*60*1000 },
  { L"5 min",5*60*1000 },
  { NULL,0 }
};

static const ControlComboBoxConfig gcSleepConfig=
  { IDC_COMBOBOX_SLEEP,offsetof(OptionsCommon,nSleep),gcaSleepList,
      L"When switched on, how long should the debug console survive closing"
      L" Winamp." };
#endif // }

////////////////
static const ControlCheckBoxConfig gcMono2StereoConfig=
  { IDC_CHECKBOX_MONO2STEREO,offsetof(OptionsCommon,bMono2Stereo),0,1,
      L"If playing mono, whether YASAPI should duplicate the one mono"
      L" channel, i.e. convert mono to stereo." };

////////////////
static const ControlCheckBoxConfig gcVolumeConfig=
  { IDC_CHECKBOX_VOLUME,offsetof(OptionsCommon,bVolume),0,1,
      L"Whether YASAPI should apply Winamp's volume control." };

#if defined (YASAPI_GAPLESS) // {
////////////////
static const ControlCheckBoxConfig gcGapless=
  { IDC_CHECKBOX_GAPLESS,offsetof(OptionsCommon,bGapless),0,1,
      L"Whether the YASAPI write method should block writing to the ring"
      L" buffer until there is enough space available." };
#elif defined (YASAPI_WRITE_BLOCK) // {
////////////////
static const ControlCheckBoxConfig gcWriteBlock=
  { IDC_CHECKBOX_WRITE_BLOCK,offsetof(OptionsCommon,bWriteBlock),0,1,
      L"Whether the YASAPI write method should block writing to the ring"
      L" buffer until there is enough space available." };
#endif // }

////////////////
static const ControlCheckBoxConfig gcFormatSupported=
  { IDC_CHECKBOX_FORMAT_SUPPORTED,
      offsetof(OptionsCommon,bFormatSupported),0,1,
      L"Whether YASAPI should call IAudioClient::IsFormatSupported before"
      L" IAudioClient::Initialize." };

////////////////
static const ControlComboBoxList gcaAudioClockList[]={
  { L"WASAPI",1 },
  { L"System",0 },
  { NULL,0 }
};

static const ControlComboBoxConfig gcAudioClockConfig=
  { IDC_COMBOBOX_CLOCK,offsetof(OptionsCommon,bAudioClock),gcaAudioClockList,
      L"Whether Winamp visualization should be synchronized by means of"
      L" the IAudioClock interface (WASAPI) or"
      L" the GetTickCount function (System)." };

////////////////
static const ControlCheckBoxConfig gcVisualizationConfig=
  { IDC_CHECKBOX_VISUALIZATION,offsetof(OptionsCommon,bVisualization),0,1,
      L"Whether the configuration dialog should visualize the ring and"
      L" the shared buffers." };

////////////////
const Control gcaCommonControlList[]={
#if defined (YASAPI_DEBUG) // {
  { &gcComboBoxType,&gcDebugConfig },
  { &gcComboBoxType,&gcSleepConfig },
#endif // }
  { &gcCheckBoxType,&gcMono2StereoConfig },
  { &gcCheckBoxType,&gcVolumeConfig },
#if defined (YASAPI_GAPLESS) // {
  { &gcCheckBoxType,&gcGapless },
#elif defined (YASAPI_WRITE_BLOCK) // {
  { &gcCheckBoxType,&gcWriteBlock },
#endif // }
  { &gcCheckBoxType,&gcFormatSupported },
  { &gcComboBoxType,&gcAudioClockConfig },
  { &gcCheckBoxType,&gcVisualizationConfig },
  { NULL,0 }
};

///////////////////////////////////////////////////////////////////////////////
static const ControlComboBoxList gcaShareModeList[]={
  { L"Share",YASAPI_SHAREMODE_SHARE },
  { L"Exclusive",YASAPI_SHAREMODE_EXCLUSIVE },
  { L"Automatic",YASAPI_SHAREMODE_AUTOMATIC },
  { NULL,0 }
};

static const ControlComboBoxConfig gcShareModeConfig=
  { IDC_COMBOBOX_SHAREMODE,offsetof(OptionsDevice,eShareMode),
      gcaShareModeList,
      L"Depending on the device, whether WASAPI should operate in share"
      L" or in exclusive mode or whether it should automatically fall back"
      L" from exclusive to share mode if needed." };

////////////////
static const ControlCheckBoxConfig gcAutoConvertPCMConfig=
  { IDC_CHECKBOX_AUTOCONVERT_PCM,offsetof(OptionsDevice,bAutoConvertPCM),
      FALSE,TRUE,
      L"In share mode, wether WASAPI should auto-convert the samplerate"
      L" to the system setting if needed." };

////////////////
static const ControlCheckBoxConfig gcSRCDefaultQualityConfig=
  { IDC_CHECKBOX_SRC_DEFAULT_QUALITY,
      offsetof(OptionsDevice,bSRCDefaultQuality),
      FALSE,TRUE,
      L"In share mode, whether WASAPI should use an improved algorithm"
      L" for sample rate conversion (SRC). Not supported by all systems." };

////////////////
static const ControlComboBoxList gcaPullList[]={
  { L"Push",FALSE },
  { L"Pull",TRUE },
  { NULL,0 }
};

static const ControlComboBoxConfig gcPullConfig=
  { IDC_COMBOBOX_PULL,offsetof(OptionsDevice,bPull),
      gcaPullList,
      L"Depending on the abilities of the device, whether WASAPI should"
      L" operate in push or pull mode." };

#if defined (YASAPI_GAPLESS) // {
static const ControlRadioButtonConfig gcGaplessOffsetList[]={
  { IDC_RADIOBUTTON_GAPLESS_OFFSET_POSITION,
      offsetof(OptionsDevice,nGaplessOffset),
      YASAPI_GAPLESS_OFFSET_POSITION,
      L"Whether the gapless offset should be measured by an 64-bit integer"
      L" representing the position." },
  { IDC_RADIOBUTTON_GAPLESS_OFFSET_TIME,
      offsetof(OptionsDevice,nGaplessOffset),
      YASAPI_GAPLESS_OFFSET_TIME,
      L"Whether the gapless offset should be measured by an floating point"
      L" value representing the time (quotinet of position and frequency)." },
  { 0,0,0,NULL }
};
#endif // }

#if defined (YASAPI_BALANCE) // {
static const ControlCheckBoxConfig gcBalanceConfig=
  { IDC_CHECKBOX_BALANCE,offsetof(OptionsDevice,bBalance),
      FALSE,TRUE,
      L"In push mode, whether YASAPI should balance the time interval"
      L" towards 0.5 of the frame buffer." };

////////////////
static const ControlSliderConfig gcBalanceStartConfig=
  { IDC_SLIDER_BALANCE_START,offsetof(OptionsDevice,qBalanceStart),
      0.0,1.0,0l,
      IDC_STATIC_BALANCE_START,
      L"Initial timer invocation after %0.2f of buffer duration",
      L"Initial time interval for balancing towards 0.5 of the frame buffer."
      L" In theory the time interval is 0.5 of the buffer duration,"
      L" in practice it may be much shorter." };
#endif // }

////////////////
const Control gcaDeviceControlList[]={
  { &gcComboBoxType,&gcShareModeConfig },
  { &gcCheckBoxType,&gcAutoConvertPCMConfig },
  { &gcCheckBoxType,&gcSRCDefaultQualityConfig },
  { &gcComboBoxType,&gcPullConfig },
#if defined (YASAPI_GAPLESS) // {
#if 0 // }
  { &gcRadioButtonType,&gcGaplessOffsetPositionConfig },
  { &gcRadioButtonType,&gcGaplessOffsetTimeConfig },
#else // } {
  { &gcRadioButtonType,gcGaplessOffsetList },
#endif // }
#endif // }
#if defined (YASAPI_BALANCE) // {
  { &gcCheckBoxType,&gcBalanceConfig },
  { &gcSliderType,&gcBalanceStartConfig },
#endif // }
  { NULL,0 }
};

const Control gcaCoreDeviceControlList[]={
  { &gcComboBoxType,&gcShareModeConfig },
  { &gcComboBoxType,&gcPullConfig },
  { NULL,0 }
};

///////////////////////////////////////////////////////////////////////////////
static const ControlComboBoxList gcaDevicePeriodList[]={
  { L"Default",YASAPI_DEVICE_PERIOD_DEFAULT },
  { L"Minimum",YASAPI_DEVICE_PERIOD_MINIMUM },
  { NULL,0 }
};

static const ControlComboBoxConfig gcDevicePeriodConfig=
  { IDC_COMBOBOX_DEVICE_PERIOD,offsetof(OptionsDevice,eDevicePeriod),
      gcaDevicePeriodList,
      L"Whether the buffer size calculation should be based on the"
      L" default or on the minimum device periode as proposed by WASAPI." };

////////////////
static const ControlSliderConfig gcShareSizeConfig=
  { IDC_SLIDER_SHARE_SIZE,offsetof(OptionsDevice,qShareSize),
      YASAPI_MIN_SHARE_SIZE,YASAPI_MAX_SHARE_SIZE,2000l,
      IDC_STATIC_SHARE_SIZE,L"%0.1f \xD7 minimum/default",
      L"A factor YASAPI should apply to the device period proposed"
      L" by WASAPI." };

static const ControlSliderCascadeList gcaBuffersList[]= {
  { IDC_SLIDER_RING_SIZE,offsetof(OptionsDevice,ring.qSize),
      IDC_STATIC_RING_SIZE,L"Size: %0.2f \xD7 size of shared buffer",
      L"The number of samples YASAPI should be able to buffer between"
      L" having received from Winamp and before sending to the device."
      L" Must be at least as much being able to cache before starting"
      L" to play." },
  { IDC_SLIDER_RING_FILL,offsetof(OptionsDevice,ring.qFill),
      IDC_STATIC_RING_FILL,L"Start Playing: %0.2f \xD7 size of shared buffer",
      L"The number of samples YASAPI should have received from Winamp before"
      L" starting to play. Must be at least the size of the buffer YASAPI"
      L" shares with the device." },
  { 0,0,0,NULL,NULL }
};

static const ControlSliderCascadeConfig gcBuffersConfig=
  { YASAPI_MIN_RING_SIZE,YASAPI_MAX_RING_SIZE,2000,gcaBuffersList };

#if defined (YASAPI_UNDERFLOW) // {
static const ControlSliderConfig gcRingUnderflowConfig=
  { IDC_SLIDER_RING_UNDERFLOW,offsetof(OptionsDevice,ring.qUnderflow),
      YASAPI_MIN_UNDERFLOW,YASAPI_MAX_UNDERFLOW,100l,
      IDC_STATIC_RING_UNDERFLOW,L"Underflow: %0.2f \xD7 size of shared buffer",
      L"The number of samples which should always remain in the ring buffer,"
      L" i.e. when YASAPI shold stop playing." };
#endif // }

////////////////
const Control gcaBuffersControlList[]={
  { &gcComboBoxType,&gcDevicePeriodConfig },
  { &gcSliderType,&gcShareSizeConfig },
  { &gcSliderCascadeType,&gcBuffersConfig },
#if defined (YASAPI_UNDERFLOW) // {
  { &gcSliderType,&gcRingUnderflowConfig },
#endif // }
  { NULL,0 }
};

==> 1.6.12&file=out_yasapi-1.6.12-src.tar.gz: out_yasapi-1.6.12/out_yasapi/yasapi_control_list.c <==
/*
 * yasapi_config_list.c
 * Copyright (C) 2015-2016 Peter Belkner <pbelkner@snafu.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <yasapi.h>
#include <resources.h>

///////////////////////////////////////////////////////////////////////////////
////////////////
static const ControlCheckBoxConfig gcMono2StereoConfig=
  { IDC_CHECKBOX_MONO2STEREO,offsetof(OptionsCommon,bMono2Stereo),0,1,
      L"If playing mono, whether YASAPI should duplicate the one mono"
      L" channel, i.e. convert mono to stereo." };

////////////////
static const ControlCheckBoxConfig gcVolumeConfig=
  { IDC_CHECKBOX_VOLUME,offsetof(OptionsCommon,bVolume),0,1,
      L"Whether YASAPI should apply Winamp's volume control." };

#if defined (YASAPI_GAPLESS) // {
////////////////
static const ControlCheckBoxConfig gcGapless=
  { IDC_CHECKBOX_GAPLESS,offsetof(OptionsCommon,bGapless),0,1,
      L"Whether playback shoud be in gapless mode." };

////////////////
static const ControlCheckBoxConfig gcDisconnect=
  { IDC_CHECKBOX_DISCONNECT,offsetof(OptionsCommon,bDisconnect),0,1,
      L"In gapless mode, whether YASAPI should automatically disconnect"
      L" the device when idle." };

////////////////
static const ControlComboBoxList gcaCheckUnderflowList[]={
  { L"100 ms",100 },
  { L"200 ms",200 },
  { L"500 ms",500 },
  { L"1 sec",1000 },
  { L"2 sec",2000 },
  { NULL,0 }
};

static const ControlComboBoxConfig gcCheckUnderflowConfig=
  { IDC_COMBOBOX_CHECK_UNDERFLOW,offsetof(OptionsCommon,nCheckUnderflow),
      gcaCheckUnderflowList,
      L"In gapless mode, the time interval after which WASAPI should"
      L" automatically disconnect the device." };
#elif defined (YASAPI_WRITE_BLOCK) // {
////////////////
static const ControlCheckBoxConfig gcWriteBlock=
  { IDC_CHECKBOX_WRITE_BLOCK,offsetof(OptionsCommon,bWriteBlock),0,1,
      L"Whether the YASAPI write method should block writing to the ring"
      L" buffer until there is enough space available." };
#endif // }

#if defined (YASAPI_SORROUND) // {
static const ControlCheckBoxConfig gcSorround=
  { IDC_CHECKBOX_SORROUND,offsetof(OptionsCommon,bSorround),0,1,
      L"Whether 4, 6, or 8 channels should be interpreted as qadrophnic"
      L" (3.1), 5.1, or 7.1 (disabled) or sorround (3.1 sorround)"
      L", 5.1 sorround, or 7.1 sorround (enabled), respectively." };
#endif // }

////////////////
static const ControlCheckBoxConfig gcFormatSupported=
  { IDC_CHECKBOX_FORMAT_SUPPORTED,
      offsetof(OptionsCommon,bFormatSupported),0,1,
      L"Whether YASAPI should call IAudioClient::IsFormatSupported before"
      L" IAudioClient::Initialize." };

////////////////
static const ControlComboBoxList gcaAudioClockList[]={
  { L"WASAPI",1 },
  { L"System",0 },
  { NULL,0 }
};

static const ControlComboBoxConfig gcAudioClockConfig=
  { IDC_COMBOBOX_CLOCK,offsetof(OptionsCommon,bAudioClock),gcaAudioClockList,
      L"Whether Winamp visualization should be synchronized by means of"
      L" the IAudioClock interface (WASAPI) or"
      L" the GetTickCount function (System)." };

////////////////
static const ControlCheckBoxConfig gcVisualizationConfig=
  { IDC_CHECKBOX_VISUALIZATION,offsetof(OptionsCommon,bVisualization),0,1,
      L"Whether the configuration dialog should visualize the ring and"
      L" the shared buffers." };

////////////////
const Control gcaCommonControls[]={
  { &gcCheckBoxType,&gcMono2StereoConfig },
  { &gcCheckBoxType,&gcVolumeConfig },
#if defined (YASAPI_GAPLESS) // {
  { &gcCheckBoxType,&gcGapless },
  { &gcCheckBoxType,&gcDisconnect },
  { &gcComboBoxType,&gcCheckUnderflowConfig },
#elif defined (YASAPI_WRITE_BLOCK) // {
  { &gcCheckBoxType,&gcWriteBlock },
#endif // }
#if defined (YASAPI_SORROUND) // {
  { &gcCheckBoxType,&gcSorround },
#endif // }
  { &gcCheckBoxType,&gcFormatSupported },
  { &gcComboBoxType,&gcAudioClockConfig },
  { &gcCheckBoxType,&gcVisualizationConfig },
  { NULL,0 }
};

///////////////////////////////////////////////////////////////////////////////
static const ControlComboBoxList gcaShareModeList[]={
  { L"Share",YASAPI_SHAREMODE_SHARE },
  { L"Exclusive",YASAPI_SHAREMODE_EXCLUSIVE },
  { L"Automatic",YASAPI_SHAREMODE_AUTOMATIC },
  { NULL,0 }
};

static const ControlComboBoxConfig gcShareModeConfig=
  { IDC_COMBOBOX_SHAREMODE,offsetof(OptionsDevice,eShareMode),
      gcaShareModeList,
      L"Depending on the device, whether WASAPI should operate in share"
      L" or in exclusive mode or whether it should automatically fall back"
      L" from exclusive to share mode if needed." };

////////////////
static const ControlCheckBoxConfig gcAutoConvertPCMConfig=
  { IDC_CHECKBOX_AUTOCONVERT_PCM,offsetof(OptionsDevice,bAutoConvertPCM),
      FALSE,TRUE,
      L"In share mode, wether WASAPI should auto-convert the samplerate"
      L" to the system setting if needed." };

////////////////
static const ControlCheckBoxConfig gcSRCDefaultQualityConfig=
  { IDC_CHECKBOX_SRC_DEFAULT_QUALITY,
      offsetof(OptionsDevice,bSRCDefaultQuality),
      FALSE,TRUE,
      L"In share mode, whether WASAPI should use an improved algorithm"
      L" for sample rate conversion (SRC). Not supported by all systems." };

////////////////
static const ControlComboBoxList gcaPullList[]={
  { L"Push",FALSE },
  { L"Pull",TRUE },
  { NULL,0 }
};

static const ControlComboBoxConfig gcPullConfig=
  { IDC_COMBOBOX_PULL,offsetof(OptionsDevice,bPull),
      gcaPullList,
      L"Depending on the abilities of the device, whether WASAPI should"
      L" operate in push or pull mode." };

#if defined (YASAPI_GAPLESS) // {
static const ControlRadioButtonConfig gcGaplessOffsetList[]={
  { IDC_RADIOBUTTON_GAPLESS_OFFSET_POSITION,
      offsetof(OptionsDevice,nGaplessOffset),
      YASAPI_GAPLESS_OFFSET_POSITION,
      L"Whether the gapless offset should be measured by an 64-bit integer"
      L" representing the position." },
  { IDC_RADIOBUTTON_GAPLESS_OFFSET_TIME,
      offsetof(OptionsDevice,nGaplessOffset),
      YASAPI_GAPLESS_OFFSET_TIME,
      L"Whether the gapless offset should be measured by an floating point"
      L" value representing the time (quotinet of position and frequency)." },
  { 0,0,0,NULL }
};
#endif // }

#if defined (YASAPI_BALANCE) // {
static const ControlCheckBoxConfig gcBalanceConfig=
  { IDC_CHECKBOX_BALANCE,offsetof(OptionsDevice,bBalance),
      FALSE,TRUE,
      L"In push mode, whether YASAPI should balance the time interval"
      L" towards 0.5 of the frame buffer." };

////////////////
static const ControlSliderConfig gcBalanceStartConfig=
  { IDC_SLIDER_BALANCE_START,offsetof(OptionsDevice,qBalanceStart),
      0.0,1.0,0l,
      IDC_STATIC_BALANCE_START,
      L"Initial timer invocation after %0.2f of buffer duration",
      L"Initial time interval for balancing towards 0.5 of the frame buffer."
      L" In theory the time interval is 0.5 of the buffer duration,"
      L" in practice it may be much shorter." };
#endif // }

////////////////
const Control gcaDeviceControls[]={
  { &gcComboBoxType,&gcShareModeConfig },
  { &gcCheckBoxType,&gcAutoConvertPCMConfig },
  { &gcCheckBoxType,&gcSRCDefaultQualityConfig },
  { &gcComboBoxType,&gcPullConfig },
#if defined (YASAPI_GAPLESS) // {
#if 0 // }
  { &gcRadioButtonType,&gcGaplessOffsetPositionConfig },
  { &gcRadioButtonType,&gcGaplessOffsetTimeConfig },
#else // } {
  { &gcRadioButtonType,gcGaplessOffsetList },
#endif // }
#endif // }
#if defined (YASAPI_BALANCE) // {
  { &gcCheckBoxType,&gcBalanceConfig },
  { &gcSliderType,&gcBalanceStartConfig },
#endif // }
  { NULL,0 }
};

const Control gcaCoreDeviceControls[]={
  { &gcComboBoxType,&gcShareModeConfig },
  { &gcComboBoxType,&gcPullConfig },
  { NULL,0 }
};

///////////////////////////////////////////////////////////////////////////////
static const ControlComboBoxList gcaDevicePeriodList[]={
  { L"Default",YASAPI_DEVICE_PERIOD_DEFAULT },
  { L"Minimum",YASAPI_DEVICE_PERIOD_MINIMUM },
  { NULL,0 }
};

static const ControlComboBoxConfig gcDevicePeriodConfig=
  { IDC_COMBOBOX_DEVICE_PERIOD,offsetof(OptionsDevice,eDevicePeriod),
      gcaDevicePeriodList,
      L"Whether the buffer size calculation should be based on the"
      L" default or on the minimum device periode as proposed by WASAPI." };

////////////////
static const ControlSliderConfig gcShareSizeConfig=
  { IDC_SLIDER_SHARE_SIZE,offsetof(OptionsDevice,qShareSize),
      YASAPI_MIN_SHARE_SIZE,YASAPI_MAX_SHARE_SIZE,2000l,
      IDC_STATIC_SHARE_SIZE,L"%0.1f \xD7 minimum/default",
      L"A factor YASAPI should apply to the device period proposed"
      L" by WASAPI." };

static const ControlSliderCascadeList gcaBuffersList[]= {
  { IDC_SLIDER_RING_SIZE,offsetof(OptionsDevice,ring.qSize),
      IDC_STATIC_RING_SIZE,L"Size: %0.2f \xD7 size of shared buffer",
      L"The number of samples YASAPI should be able to buffer between"
      L" having received from Winamp and before sending to the device."
      L" Must be at least as much being able to cache before starting"
      L" to play." },
  { IDC_SLIDER_RING_FILL,offsetof(OptionsDevice,ring.qFill),
      IDC_STATIC_RING_FILL,L"Start Playing: %0.2f \xD7 size of shared buffer",
      L"The number of samples YASAPI should have received from Winamp before"
      L" starting to play. Must be at least the size of the buffer YASAPI"
      L" shares with the device." },
  { 0,0,0,NULL,NULL }
};

static const ControlSliderCascadeConfig gcBuffersConfig=
  { YASAPI_MIN_RING_SIZE,YASAPI_MAX_RING_SIZE,2000,gcaBuffersList };

#if defined (YASAPI_UNDERFLOW) // {
static const ControlSliderConfig gcRingUnderflowConfig=
  { IDC_SLIDER_RING_UNDERFLOW,offsetof(OptionsDevice,ring.qUnderflow),
      YASAPI_MIN_UNDERFLOW,YASAPI_MAX_UNDERFLOW,100l,
      IDC_STATIC_RING_UNDERFLOW,L"Underflow: %0.2f \xD7 size of shared buffer",
      L"The number of samples which should always remain in the ring buffer,"
      L" i.e. when YASAPI shold stop playing." };
#endif // }

////////////////
const Control gcaBuffersControls[]={
  { &gcComboBoxType,&gcDevicePeriodConfig },
  { &gcSliderType,&gcShareSizeConfig },
  { &gcSliderCascadeType,&gcBuffersConfig },
#if defined (YASAPI_UNDERFLOW) // {
  { &gcSliderType,&gcRingUnderflowConfig },
#endif // }
  { NULL,0 }
};
